from game import *
from crash import *
import math
import random

image_player = image('player_spaceship.png')
image_enemy = image('enemy_spaceship.png')
image_weapon = image('homing.png')

SW, SH = 1, 9/16

def player(p):
    v = 0.02
    if key(LEFT):
        p.x -= v
    if key(RIGHT):
        p.x += v
    if key(UP):
        p.y += v
    if key(DOWN):
        p.y -= v
    p.x = max(-SW+p.sx, min(SW-p.sx, p.x))
    p.y = max(-SH+p.sy, min(SH-p.sy, p.y))
    for e in group(enemy):
        d = math.dist((p.x, p.y), (e.x, e.y))
        if d < e.sx or e.x < -SW:
            new_crash(p.x, p.y, 0.01, 20, 0.98)
            p.life = 0
            for l in group(laser):
                l.parent = None
    p.time -= 1
    if key(Z) and p.time <= 0:
        new_shot(p.x, p.y, 0.05, 0, 3, 0.04)
        p.time = 5
    elif key(X):
        add(laser, image_weapon, 0.05, p.x, p.y, 0.07, parent=p)
    elif key(C) and p.time <= 0:
        add(homing_laser, image_weapon, 0.05, p.x, p.y, 0.03, life=10)
        p.time = 30

def enemy(e):
    e.x += e.vx
    e.y += e.vy
    e.r += 0.01
    for w in group(shot, laser, homing_laser):
        d = math.dist((w.x, w.y), (e.x, e.y))
        if d < e.sx*2:
            e.life -= w.life
            w.life = 0
    if e.life <= 0:
        new_crash(e.x, e.y, 0.01, 20, 0.95)
    if abs(e.x) > SW+e.sx or abs(e.y) > SH+e.sy:
        e.life = 0

def new_shot(x, y, v, dir, n, angle):
    for i in range(n):
        if n > 1:
            r = (dir+angle/(n-1)*(i-(n-1)/2))
        else:
            r = dir
        rad = r*math.pi*2
        vx = math.cos(rad)*v
        vy = math.sin(rad)*v
        add(shot, image_weapon, 0.05, x, y, vx, vy, r=r)

def shot(s):
    s.x += s.vx
    s.y += s.vy
    if abs(s.x) > SW+s.sx or abs(s.y) > SH+s.sx:
        s.life = 0

def laser(l):
    l.x += l.vx
    if l.parent:
        l.y = l.parent.y
    if abs(l.x) > SW+l.sx or abs(l.y) > SH+l.sx:
        l.life = 0

def homing_laser(h):
    target = None
    target_d = float('inf')
    for e in group(enemy):
        d = math.dist((h.x, h.y), (e.x, e.y))
        if d < target_d:
            target, target_d = e, d
    vdir = math.atan2(h.vy, h.vx)/math.pi/2
    if target:
        tdir = math.atan2(target.y-h.y, target.x-h.x)/math.pi/2
        if vdir-tdir > 0.5:
            tdir += 1
        if tdir-vdir > 0.5:
            tdir -= 1
        vdir += (tdir-vdir)*0.1
        v = math.dist((h.vx, h.vy), (0, 0))
        rad = vdir*math.pi*2
        h.vx = math.cos(rad)*v
        h.vy = math.sin(rad)*v
    h.x += h.vx
    h.y += h.vy
    h.r = vdir
    if h.time%2 == 0:
        add(homing_tail, image_weapon, h.sx, h.x, h.y, r=h.r)
    h.time += 1
    if abs(h.x) > SW+h.sx or abs(h.y) > SH+h.sy:
        h.life = 0

def homing_tail(h):
    h.sy *= 0.95
    if h.sy < 0.01:
        h.life = 0

def stage(s):
    if random.random() < 0.05:
        size = 0.08
        x = SW+size
        y = random.uniform(-SH+size, SH-size)
        add(enemy, image_enemy, size, x, y, -0.005, life=10)

def start():
    add(player, image_player, 0.08, -0.9, 0)
    add(stage)

run(start, 1280, 720)
